from typing import Any, List

from fastapi import APIRouter, Depends, HTTPException, status
from sqlalchemy.orm import Session

from app import crud
from app.api import deps
from app.models.user import User
from app.schemas.tag import Tag, TagCreate, TagUpdate

router = APIRouter()


@router.get("/", response_model=List[Tag])
def read_tags(
    db: Session = Depends(deps.get_db),
    skip: int = 0,
    limit: int = 100,
) -> Any:
    """
    Retrieve tags.
    """
    tags = crud.tag.get_multi(db, skip=skip, limit=limit)
    return tags


@router.post("/", response_model=Tag)
def create_tag(
    *,
    db: Session = Depends(deps.get_db),
    tag_in: TagCreate,
    current_user: User = Depends(deps.get_current_active_user),
) -> Any:
    """
    Create new tag.
    """
    # Check if tag already exists
    tag = crud.tag.get_by_name(db=db, name=tag_in.name)
    if tag:
        raise HTTPException(
            status_code=status.HTTP_400_BAD_REQUEST,
            detail="Tag with this name already exists",
        )
    
    tag = crud.tag.create(db=db, obj_in=tag_in)
    return tag


@router.get("/{id}", response_model=Tag)
def read_tag(
    *,
    db: Session = Depends(deps.get_db),
    id: int,
) -> Any:
    """
    Get tag by ID.
    """
    tag = crud.tag.get(db=db, id=id)
    if not tag:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND, detail="Tag not found"
        )
    return tag


@router.put("/{id}", response_model=Tag)
def update_tag(
    *,
    db: Session = Depends(deps.get_db),
    id: int,
    tag_in: TagUpdate,
    current_user: User = Depends(deps.get_current_active_user),
) -> Any:
    """
    Update a tag.
    """
    tag = crud.tag.get(db=db, id=id)
    if not tag:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND, detail="Tag not found"
        )
    
    # Check if new tag name already exists
    if tag_in.name and tag_in.name != tag.name:
        existing_tag = crud.tag.get_by_name(db=db, name=tag_in.name)
        if existing_tag:
            raise HTTPException(
                status_code=status.HTTP_400_BAD_REQUEST,
                detail="Tag with this name already exists",
            )
    
    tag = crud.tag.update(db=db, db_obj=tag, obj_in=tag_in)
    return tag


@router.delete("/{id}", status_code=status.HTTP_204_NO_CONTENT, response_model=None)
def delete_tag(
    *,
    db: Session = Depends(deps.get_db),
    id: int,
    current_user: User = Depends(deps.get_current_active_user),
) -> Any:
    """
    Delete a tag.
    """
    tag = crud.tag.get(db=db, id=id)
    if not tag:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND, detail="Tag not found"
        )
    crud.tag.remove(db=db, id=id)
    return None